/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include <iostream>
#include <gtkmm/cellrenderercombo.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/label.h>
#include <gtkmm/eventbox.h>
#include <gdk/gdkkeysyms.h>

#include <gtkmm/textview.h>
#include <gtkmm/celleditable.h>
#include <gtkmm/spinbutton.h>

#include "Config.h"
#include "debug.h"

#include "SubtitleModifier.h"
#include "Document.h"
#include "SubtitleView.h"
#include "utility.h"
#include "ActionSystem.h"

/*
 *	une CellEditable deriver de TextView qui permet d'avoir plusieur ligne
 *	contrairement a GtkEntry (par defaut)
 */
class TextViewCell : public Gtk::TextView, public Gtk::CellEditable
{
public:
	/*
	 *
	 */
	TextViewCell(const Glib::ustring &path)
	:
		Glib::ObjectBase(typeid(TextViewCell)),
		Gtk::CellEditable(),
		m_path(path)
	{
		bool center;
		Config::getInstance().get_value_bool("subtitle-view", "property-alignment-center", center, false);
	
		if(center)
		{
			set_justification(Gtk::JUSTIFY_CENTER);
		}
	}

	/*
	 *
	 */
	Glib::ustring get_path()
	{
		return m_path;
	}

	/*
	 *
	 */
	Glib::ustring get_text() 
	{
		Glib::RefPtr<Gtk::TextBuffer> buffer = get_buffer();

		Gtk::TextBuffer::iterator start, end;

		buffer->get_bounds(start,end);
		return buffer->get_text(start,end);
	}

	/*
	 *
	 */
	void set_text(const Glib::ustring &text)
	{
		get_buffer()->set_text(text);
	}

	/*
	 *
	 */
	bool on_key_press_event(GdkEventKey* event)
	{
		if(event->keyval == GDK_Escape)
		{
			remove_widget();
			return true;
		}
		else if( (
				 event->keyval == GDK_Return ||  
				 event->keyval == GDK_KP_Enter ||  
				 event->keyval == GDK_ISO_Enter ||  
				 event->keyval == GDK_3270_Enter 
				)	&& (event->state & GDK_CONTROL_MASK))
		{
			editing_done();
			remove_widget();
			return true;
		}
		return Gtk::TextView::on_key_press_event(event);
	}

protected:
	Glib::ustring m_path;
};


/*
 *	derive simplement de CellRendererText pour pouvoir utiliser
 *	TextViewCell a l'edition.
 */
class CellRendererTextMultiline : public Gtk::CellRendererText
{
public:
	/*
	 *
	 */
	CellRendererTextMultiline()
	:
		Glib::ObjectBase(typeid(CellRendererTextMultiline)),
		Gtk::CellRendererText(),
		m_textViewCell(NULL), m_document(NULL)
	{
	}
	
	void setDocument(Document *doc)
	{
		m_document = doc;
	}

	/*
	 *
	 */
	
	/*
	 *
	 */
	virtual Gtk::CellEditable* start_editing_vfunc(
			GdkEvent* event, 
			Gtk::Widget &widget, 
			const Glib::ustring &path, 
			const Gdk::Rectangle& background_area,
			const Gdk::Rectangle& cell_area,
			Gtk::CellRendererState flags)
	{
		if(!property_editable())
			return NULL;

		m_textViewCell = manage(new TextViewCell(path));
		m_textViewCell->set_wrap_mode(Gtk::WRAP_NONE);
		m_textViewCell->set_text(property_text());
		m_textViewCell->set_size_request(cell_area.get_width(), cell_area.get_height());

		m_textViewCell->signal_editing_done().connect(
				sigc::mem_fun(*this, &CellRendererTextMultiline::cell_editing_done));

		m_textViewCell->grab_focus();
		
		m_textViewCell->show();

		if(m_document)
			m_document->flash_message(_("Use Ctrl+Return for exit and Return for line-break"));
		
		return m_textViewCell;
	}

	/*
	 *
	 */
	void cell_editing_done()
	{
		if(m_textViewCell)
			edited(m_textViewCell->get_path(), m_textViewCell->get_text());
	}

protected:
	TextViewCell* m_textViewCell;
	Document* m_document;
};



/*
 *
 */
SubtitleView::SubtitleView(Document &doc)
{
	m_refDocument = &doc;

	m_subtitleModel = m_refDocument->m_subtitleModel;
	m_styleModel = m_refDocument->m_styleModel;
	
#warning "FIXME: connect after document"
	set_model(m_subtitleModel);

	createColumnNum();
	createColumnLayer();
	createColumnStart();
	createColumnEnd();
	createColumnDuration();
	createColumnStyle();
	createColumnName();
	
	createColumnMarginL();
	createColumnMarginR();
	createColumnMarginV();
	
	createColumnEffect();

	createColumnText();
	//createColumnCharactersPerLine();

	createColumnTranslation();

	createColumnNote();

	set_rules_hint(true);
	set_enable_search(false);
	set_search_column(m_column.num);

	// config
	loadCfg();


	get_selection()->signal_changed().connect(
			sigc::mem_fun(*this, &SubtitleView::on_selection_changed));

	get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);

	ActionSystem::getInstance().signal_emit().connect(
			sigc::mem_fun(*this, &SubtitleView::on_execute_action));

	Config::getInstance().signal_changed("subtitle-view").connect(
			sigc::mem_fun(*this, &SubtitleView::on_config_subtitle_view_changed));
}

/*
 *
 */
SubtitleView::~SubtitleView()
{
	//saveCfg();
}

/*
 *
 */
void SubtitleView::loadCfg()
{
	se_debug(SE_DEBUG_VIEW);

	bool state = false;
	
	Config &cfg = Config::getInstance();

#define LOAD_CONFIG(name, default_value) \
	cfg.get_value_bool("subtitle-view", "show-column-"name, state, default_value);	set_column_visible(name, state);

	LOAD_CONFIG("number", true);
	LOAD_CONFIG("layer", false);
	LOAD_CONFIG("start", true);
	LOAD_CONFIG("end", true);
	LOAD_CONFIG("duration", true);
	LOAD_CONFIG("style", false);
	LOAD_CONFIG("name", false);
	LOAD_CONFIG("margin-l", false);
	LOAD_CONFIG("margin-r", false);
	LOAD_CONFIG("margin-v", false);
	LOAD_CONFIG("effect", false);
	LOAD_CONFIG("text", true);
	LOAD_CONFIG("translation", false);
	//LOAD_CONFIG("characters-per-line", false);
	LOAD_CONFIG("note", false);
	
#undef LOAD_CONFIG
}

/*
 *
 */
void SubtitleView::saveCfg()
{
	se_debug(SE_DEBUG_VIEW);

	Config &cfg = Config::getInstance();

#define SET(name)	cfg.set_value_bool("subtitle-view", "show-column-"name, get_column_visible(name));

	SET("number");
	SET("layer");
	SET("start");
	SET("end");
	SET("duration");
	SET("style");
	SET("name");
	SET("margin-l");
	SET("margin-r");
	SET("margin-v");
	SET("effect");
	SET("text");
	SET("translation");
	//SET("characters-per-line");
	SET("note");
	
#undef SET
}

/*
 *
 */
void SubtitleView::set_tooltips(Gtk::TreeViewColumn *column, const Glib::ustring &text)
{
	se_debug_message(SE_DEBUG_VIEW, "[%s]=%s", column->get_title().c_str(), text.c_str());
/*
	Gtk::EventBox *event = manage(new Gtk::EventBox);

	Gtk::Label* label = manage(new Gtk::Label(column->get_title()));
	event->add(*label);

	Gtk::Tooltips *tooltips = manage(new Gtk::Tooltips);
	tooltips->set_tip(*event, text);
	tooltips->enable();

	column->set_widget(*event);
	event->show_all();
*/
}

/*
 *	create columns
 */
void SubtitleView::createColumnNum()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("num")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.num);
	
	append_column(*column);
	//append_column_numeric_editable("num", m_column.num, "%d");
	renderer->property_editable() = false;
	renderer->property_yalign() = 0;
	
	//renderer->property_visible() = false;

	m_columns["number"] = column;

	set_tooltips(column, _("This number column"));
}

/*
 *
 */
void SubtitleView::createColumnLayer()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("layer")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.layer);

	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_layer));

	append_column(*column);

	m_columns["layer"] = column;

	set_tooltips(column, _("Layer number."));
}

/*
 *
 */
void SubtitleView::createColumnStart()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("start")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.start);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_start));

	append_column(*column);

	m_columns["start"] = column;

	set_tooltips(column, _("This time is the time when a subtitle appears on the screen."));
}

/*
 *
 */
void SubtitleView::createColumnEnd()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("end")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.end);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_end));

	append_column(*column);

	m_columns["end"] = column;

	set_tooltips(column, _("This time is the time when a subtitle disappears from the screen."));
}

/*
 *
 */
void SubtitleView::createColumnDuration()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("duration")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.duration);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_duration));

	append_column(*column);

	m_columns["duration"] = column;
#warning "FIXME tooltip duration"
	set_tooltips(column, _("This time is the duration..."));
}

/*
 *
 */
void SubtitleView::createColumnStyle()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererCombo* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("style")));
	renderer = manage(new Gtk::CellRendererCombo);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.style);
	
	renderer->property_model() =	m_styleModel;
	renderer->property_text_column() = 0;
	renderer->property_editable() = true;
	renderer->property_has_entry() = false;
	renderer->property_yalign() = 0;
	
	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_style));

	append_column(*column);

	m_columns["style"] = column;
}

/*
 *
 */
void SubtitleView::createColumnName()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("name")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.name);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_name));
	
	append_column(*column);

	m_columns["name"] = column;
}

/*
 *
 */
void SubtitleView::createColumnText()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	column = manage(new Gtk::TreeViewColumn(_("text")));
	append_column(*column);

	// text
	{
		CellRendererTextMultiline* renderer = NULL;
		renderer = manage(new CellRendererTextMultiline);
		renderer->setDocument(m_refDocument);

		column->pack_start(*renderer, true);//false
		column->add_attribute(renderer->property_text(), m_column.text);
	
		renderer->property_editable() = true;
		renderer->property_yalign() = 0;
	
		renderer->signal_edited().connect(
				sigc::mem_fun(*this, &SubtitleView::on_edited_text));
		
		bool center;
		Config::getInstance().get_value_bool("subtitle-view", "property-alignment-center", center, false);
	
		if(center)
		{
			renderer->property_xalign() = 0.5;
			renderer->property_alignment() = Pango::ALIGN_CENTER;
		}
	}
	// cpl
	{
		Gtk::CellRendererText* renderer = NULL;
		renderer = manage(new Gtk::CellRendererText);
	
		column->pack_start(*renderer, false);
		column->add_attribute(renderer->property_text(), m_column.characters_per_line_text);
		renderer->property_yalign() = 0;
		//renderer->property_style() = Pango::STYLE_ITALIC;
		renderer->property_weight() = Pango::WEIGHT_ULTRALIGHT;
		//renderer->property_attributes() = 

		bool show=true;
		
		Config::getInstance().get_value_bool("subtitle-view", "show-character-per-line", show, true);

		renderer->property_visible() = show;
	}


	column->set_resizable(true);
	//column->set_expand(true);

	m_columns["text"] = column;
}

/*
 *
 */
void SubtitleView::createColumnTranslation()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	column = manage(new Gtk::TreeViewColumn(_("translation")));

	//translation
	{
		CellRendererTextMultiline* renderer = NULL;
	
		renderer = manage(new CellRendererTextMultiline);
		renderer->setDocument(m_refDocument);

		column->pack_start(*renderer, true);
		column->add_attribute(renderer->property_text(), m_column.translation);
	
		append_column(*column);
	
		renderer->property_editable() = true;
		renderer->property_yalign() = 0;

		bool center;
		Config::getInstance().get_value_bool("subtitle-view", "property-alignment-center", center, false);

		if(center)
		{
			renderer->property_xalign() = 0.5;
			renderer->property_alignment() = Pango::ALIGN_CENTER;
		}
			
		renderer->signal_edited().connect(
			sigc::mem_fun(*this, &SubtitleView::on_edited_translation));
	}
	// cpl
	{
		Gtk::CellRendererText* renderer = NULL;
		renderer = manage(new Gtk::CellRendererText);
	
		column->pack_end(*renderer, false);
		column->add_attribute(renderer->property_text(), m_column.characters_per_line_translation);
		renderer->property_yalign() = 0;
		//renderer->property_style() = Pango::STYLE_ITALIC;
		renderer->property_weight() = Pango::WEIGHT_ULTRALIGHT;
		//renderer->property_attributes() = 
		bool show=true;
		Config::getInstance().get_value_bool("subtitle-view", "show-character-per-line", show, true);

		renderer->property_visible() = show;
	}

	column->set_resizable(true);
//	column->set_expand(true);

	m_columns["translation"] = column;
}

/*
 *
 */
void SubtitleView::createColumnNote()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	CellRendererTextMultiline* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("note")));
	renderer = manage(new CellRendererTextMultiline);
	renderer->setDocument(m_refDocument);

	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.note);
	
	append_column(*column);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_note));

	column->set_resizable(true);
//	column->set_expand(true);

	m_columns["note"] = column;
}

/*
 *
 */
void SubtitleView::createColumnEffect()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("effect")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.effect);
	
	append_column(*column);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_effect));

	column->set_resizable(true);
//	column->set_expand(true);

	m_columns["effect"] = column;
}

/*
 *
 */
void SubtitleView::createColumnMarginR()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("R")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.marginR);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_margin_r));

	append_column(*column);

	m_columns["margin-r"] = column;
}

/*
 *
 */
void SubtitleView::createColumnMarginL()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("L")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.marginL);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_margin_l));

	append_column(*column);

	m_columns["margin-l"] = column;
}

/*
 *
 */
void SubtitleView::createColumnMarginV()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("V")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.marginV);
	
	renderer->property_editable() = true;
	renderer->property_yalign() = 0;

	renderer->signal_edited().connect(
		sigc::mem_fun(*this, &SubtitleView::on_edited_margin_v));

	append_column(*column);

	m_columns["margin-v"] = column;
}

/*
 *
 */
/*
void SubtitleView::createColumnCharactersPerLine()
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn* column = NULL;
	Gtk::CellRendererText* renderer = NULL;
	
	column = manage(new Gtk::TreeViewColumn(_("cpl")));
	renderer = manage(new Gtk::CellRendererText);
	
	column->pack_end(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.characters_per_line_text);
	
	//renderer->property_editable() = true;
	renderer->property_yalign() = 0;
	//renderer->signal_edited().connect(
	//	sigc::mem_fun(*this, &SubtitleView::on_edited_));

	append_column(*column);

	m_columns["characters-per-line"] = column;

	set_tooltips(column, _("This column represents the number of characters per line."));
}
*/

/*
 *	retourne l'item select ou NULL
 */
Gtk::TreeIter SubtitleView::getSelected()
{
	se_debug(SE_DEBUG_VIEW);
	
	Glib::RefPtr<Gtk::TreeSelection> selection = get_selection();
	
	std::vector<Gtk::TreeModel::Path> rows = selection->get_selected_rows();

	if(rows.size() > 0)
	{
		return selection->get_model()->get_iter(rows[0]);
	}

	Gtk::TreeIter null;
	return null;
}

/*
 *
 */
void SubtitleView::on_selection_changed()
{
	se_debug(SE_DEBUG_VIEW);

	m_refDocument->emit_signal("subtitle-selection-changed");
}

/*
 * 
 */
void SubtitleView::on_edited_layer( const Glib::ustring &path, const Glib::ustring &value)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), value.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		unsigned int val = 0;
		if(from_string(value, val))
			subtitle.set_layer(value);
	}
}


/*
 *	callback utiliser pour modifier le temps directement depuis la list (treeview)
 */
void SubtitleView::on_edited_start( const Glib::ustring &path, 
																		const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		if(SubtitleTime::validate(newtext))
		{
			subtitle.set_start(newtext);
			
			m_refDocument->emit_signal("subtitle-time-changed");
		}
		else
		{
			std::cerr << "on_edited_start::invalide> " << newtext << std::endl;
		}
	}
}

/*
 *	callback utiliser pour modifier le temps directement depuis la list (treeview)
 */
void SubtitleView::on_edited_end( const Glib::ustring &path, 
																	const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		if(SubtitleTime::validate(newtext))
		{
			subtitle.set_end(newtext);

			m_refDocument->emit_signal("subtitle-time-changed");
		}
		else
		{
			std::cerr << "on_edited_end::invalide> " << newtext << std::endl;
		}
	}
}

/*
 *	callback utiliser pour modifier le temps directement depuis la list (treeview)
 */
void SubtitleView::on_edited_duration( const Glib::ustring &path, 
																	const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		if(SubtitleTime::validate(newtext))
		{
			subtitle.set_duration(newtext);

			m_refDocument->emit_signal("subtitle-time-changed");
		}
		else
		{
			std::cerr << "on_edited_duration::invalide> " << newtext << std::endl;
		}
	}
}

/*
 *	callback utiliser pour modifier le texte
 */
void SubtitleView::on_edited_text( const Glib::ustring &path, 
																		const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		subtitle.set_text(newtext);
	}
}

/*
 *	callback utiliser pour modifier le texte
 */
void SubtitleView::on_edited_translation( const Glib::ustring &path, 
																		const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		subtitle.set_translation(newtext);
	}
}

/*
 *	callback utiliser pour modifier le texte
 */
void SubtitleView::on_edited_note( const Glib::ustring &path, 
																		const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		subtitle.set_note(newtext);
	}
}

/*
 */
void SubtitleView::on_edited_effect( const Glib::ustring &path, 
																		const Glib::ustring &newtext)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newtext.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		subtitle.set_effect(newtext);
	}
}


/*
 *	callback utiliser pour modifier le style a partir d'un menu
 */
void SubtitleView::on_edited_style( const Glib::ustring &path, 
																		const Glib::ustring &newstyle)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newstyle.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		subtitle.set_style(newstyle);
	}
}

/*
 *	callback utiliser pour modifier le nom
 */
void SubtitleView::on_edited_name( const Glib::ustring &path, 
																		const Glib::ustring &newname)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), newname.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		subtitle.set_name(newname);
	}
}

/*
 * 
 */
void SubtitleView::on_edited_margin_l( const Glib::ustring &path, const Glib::ustring &value)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), value.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		unsigned int val = 0;
		if(from_string(value, val))
			subtitle.set_margin_l(value);
	}
}

/*
 * 
 */
void SubtitleView::on_edited_margin_r( const Glib::ustring &path, const Glib::ustring &value)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), value.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		unsigned int val = 0;
		if(from_string(value, val))
			subtitle.set_margin_r(value);
	}
}

/*
 * 
 */
void SubtitleView::on_edited_margin_v( const Glib::ustring &path, const Glib::ustring &value)
{
	se_debug_message(SE_DEBUG_VIEW, "%s %s", path.c_str(), value.c_str());

	SubtitleModifier subtitle(m_subtitleModel->get_iter(path));
	if(subtitle)
	{
		unsigned int val = 0;
		if(from_string(value, val))
			subtitle.set_margin_v(value);
	}
}

/*
 *
 */
void SubtitleView::select_and_set_cursor(const Gtk::TreeIter &iter)
{
	se_debug(SE_DEBUG_VIEW);

	Gtk::TreeViewColumn *column = NULL;
	Gtk::TreeModel::Path path;
	get_cursor(path, column);

	if(column == NULL)
		column = m_columns["text"];

	get_selection()->select(iter);
	set_cursor(m_subtitleModel->get_path(iter), *column, false);
}



/*
 *
 */
void SubtitleView::set_column_visible(const Glib::ustring& name, bool state)
{
	se_debug_message(SE_DEBUG_VIEW, "%s=%s", name.c_str(), state ? "true" : "false");

	std::map<Glib::ustring, Gtk::TreeViewColumn*>::iterator it = m_columns.find(name);

	if(it != m_columns.end())
		it->second->set_visible(state);
	else
		se_debug_message(SE_DEBUG_VIEW, "column <%s> not found.", name.c_str());
}

/*
 *
 */
bool SubtitleView::get_column_visible(const Glib::ustring &name)
{
	std::map<Glib::ustring, Gtk::TreeViewColumn*>::iterator it = m_columns.find(name);
	if(it == m_columns.end())
	{
		se_debug_message(SE_DEBUG_VIEW, "<%s> not found.", name.c_str());
		return false;
	}
	
	se_debug_message(SE_DEBUG_VIEW, "<%s> = %s", name.c_str(), it->second->get_visible() ? "true" : "false");
	return it->second->get_visible();
}

/*
 *
 */
bool SubtitleView::on_key_press_event(GdkEventKey *event)
{
	bool is_num = utility::is_num(event->string);

	// seulement si c'est different on met à jour
	if(is_num != get_enable_search())
		set_enable_search(is_num);
	
	return Gtk::TreeView::on_key_press_event(event);
}

/*
 *
 */
void SubtitleView::on_config_subtitle_view_changed(const Glib::ustring &key, const Glib::ustring &value)
{
	if(key.find("show-column-") != Glib::ustring::npos)
	{
		bool state;
	
		Glib::ustring column = key;
		column.erase(0, Glib::ustring("show-column-").size());

		if(from_string(value, state))
		{
			set_column_visible(column, state);
		}
	}
	else if(key == "property-alignment-center")
	{
		bool state;
		if(from_string(value, state))
		{
			Gtk::CellRendererText *renderer = NULL;
			
			renderer = dynamic_cast<Gtk::CellRendererText*>(m_columns["text"]->get_first_cell_renderer());
			renderer->property_xalign() = state ? 0.5 : 0.0;
			renderer->property_alignment() = state ? Pango::ALIGN_CENTER : Pango::ALIGN_LEFT;

			renderer = dynamic_cast<Gtk::CellRendererText*>(m_columns["translation"]->get_first_cell_renderer());
			renderer->property_xalign() = state ? 0.5 : 0.0;
			renderer->property_alignment() = state ? Pango::ALIGN_CENTER : Pango::ALIGN_LEFT;
		}

		queue_draw();
	}
	else if(key == "show-character-per-line")
	{
		bool state;
		if(from_string(value, state))
		{
			std::vector<Gtk::CellRenderer*> cells;
			
			cells = m_columns["text"]->get_cell_renderers();
			cells[1]->property_visible() = state;

			cells = m_columns["translation"]->get_cell_renderers();
			cells[1]->property_visible() = state;
		}
		queue_draw();
	}
}

/*
 *
 */
void SubtitleView::on_execute_action(const Glib::ustring &action)
{
	if(action.find("view/") != Glib::ustring::npos)
	{
		Config &cfg = Config::getInstance();

#define SET_COLUMN(name, value) if(get_column_visible(name) != value) cfg.set_value_bool("subtitle-view", "show-column-"name, value);

		if(action == "view/simple")
		{
			SET_COLUMN("number", true);
			SET_COLUMN("layer", false);
			SET_COLUMN("start", true);
			SET_COLUMN("end", true);
			SET_COLUMN("duration", true);
			SET_COLUMN("style", false);
			SET_COLUMN("name", false);
			SET_COLUMN("margin-r", false);
			SET_COLUMN("margin-l", false);
			SET_COLUMN("margin-v", false);
			SET_COLUMN("effect", false);
			SET_COLUMN("text", true);
			SET_COLUMN("translation", false);
			//SET_COLUMN("characters-per-line", false);
			SET_COLUMN("note", false);
		}
		else if(action == "view/advanced")
		{
			SET_COLUMN("number", true);
			SET_COLUMN("layer", true);
			SET_COLUMN("start", true);
			SET_COLUMN("end", true);
			SET_COLUMN("duration", true);
			SET_COLUMN("style", true);
			SET_COLUMN("name", true);
			SET_COLUMN("margin-r", false);
			SET_COLUMN("margin-l", false);
			SET_COLUMN("margin-v", false);
			SET_COLUMN("effect", false);
			SET_COLUMN("text", true);
			SET_COLUMN("translation", false);
			//SET_COLUMN("characters-per-line", true);
			SET_COLUMN("note", false);
		}
		else if(action == "view/translation")
		{
			SET_COLUMN("number", true);
			SET_COLUMN("layer", false);
			SET_COLUMN("start", false);
			SET_COLUMN("end", false);
			SET_COLUMN("duration", false);
			SET_COLUMN("style", false);
			SET_COLUMN("name", false);
			SET_COLUMN("margin-r", false);
			SET_COLUMN("margin-l", false);
			SET_COLUMN("margin-v", false);
			SET_COLUMN("effect", false);
			SET_COLUMN("text", true);
			SET_COLUMN("translation", true);
			//SET_COLUMN("characters-per-line", false);
			SET_COLUMN("note", false);
		}
		else if(action == "view/timing")
		{
			SET_COLUMN("number", true);
			SET_COLUMN("layer", false);
			SET_COLUMN("start", true);
			SET_COLUMN("end", true);
			SET_COLUMN("duration", true);
			SET_COLUMN("style", false);
			SET_COLUMN("name", false);
			SET_COLUMN("margin-r", false);
			SET_COLUMN("margin-l", false);
			SET_COLUMN("margin-v", false);
			SET_COLUMN("effect", false);
			SET_COLUMN("text", true);
			SET_COLUMN("translation", false);
			//SET_COLUMN("characters-per-line", true);
			SET_COLUMN("note", false);
		}

#undef SET_COLUMN

	}
}
